@@ -152,12 +152,7 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
 
         // Write all structs
         for (handle, ty) in module.types.iter() {
-            if let TypeInner::Struct {
-                top_level,
-                ref members,
-                ..
-            } = ty.inner
-            {
+            if let TypeInner::Struct { ref members, .. } = ty.inner {
                 if let Some(member) = members.last() {
                     if let TypeInner::Array {
                         size: crate::ArraySize::Dynamic,
@@ -180,7 +175,6 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
                 self.write_struct(
                     module,
                     handle,
-                    top_level,
                     members,
                     ep_result.map(|r| (r.0, Io::Output)),
                 )?;
@@ -703,7 +697,6 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
         &mut self,
         module: &Module,
         handle: Handle<crate::Type>,
-        _block: bool,
         members: &[crate::StructMember],
         shader_stage: Option<(ShaderStage, Io)>,
     ) -> BackendResult {
@@ -1358,7 +1351,6 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
             Statement::Switch {
                 selector,
                 ref cases,
-                ref default,
             } => {
                 // Start the switch
                 write!(self.out, "{}", level)?;
@@ -1378,11 +1370,16 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
                 let indent_level_2 = indent_level_1.next();
 
                 for case in cases {
-                    writeln!(
-                        self.out,
-                        "{}case {}{}: {{",
-                        indent_level_1, case.value, type_postfix
-                    )?;
+                    match case.value {
+                        crate::SwitchValue::Integer(value) => writeln!(
+                            self.out,
+                            "{}case {}{}: {{",
+                            indent_level_1, value, type_postfix
+                        )?,
+                        crate::SwitchValue::Default => {
+                            writeln!(self.out, "{}default: {{", indent_level_1)?
+                        }
+                    }
 
                     if case.fall_through {
                         // Generate each fallthrough case statement in a new block. This is done to
@@ -1401,25 +1398,13 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
 
                     if case.fall_through {
                         writeln!(self.out, "{}}}", indent_level_2)?;
-                    } else {
+                    } else if case.body.last().map_or(true, |s| !s.is_terminator()) {
                         writeln!(self.out, "{}break;", indent_level_2)?;
                     }
 
                     writeln!(self.out, "{}}}", indent_level_1)?;
                 }
 
-                // Only write the default block if the block isn't empty
-                // Writing default without a block is valid but it's more readable this way
-                if !default.is_empty() {
-                    writeln!(self.out, "{}default: {{", indent_level_1)?;
-
-                    for sta in default {
-                        self.write_stmt(module, sta, func_ctx, indent_level_2)?;
-                    }
-
-                    writeln!(self.out, "{}}}", indent_level_1)?;
-                }
-
                 writeln!(self.out, "{}}}", level)?
             }
         }
@@ -1609,6 +1594,7 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
             Expression::ImageSample {
                 image,
                 sampler,
+                gather,
                 coordinate,
                 array_index,
                 offset,
@@ -1616,23 +1602,30 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
                 depth_ref,
             } => {
                 use crate::SampleLevel as Sl;
+                const COMPONENTS: [&str; 4] = ["", "Green", "Blue", "Alpha"];
 
-                let texture_func = match level {
-                    Sl::Auto => {
-                        if depth_ref.is_some() {
-                            "SampleCmp"
-                        } else {
-                            "Sample"
-                        }
-                    }
-                    Sl::Zero => "SampleCmpLevelZero",
-                    Sl::Exact(_) => "SampleLevel",
-                    Sl::Bias(_) => "SampleBias",
-                    Sl::Gradient { .. } => "SampleGrad",
+                let (base_str, component_str) = match gather {
+                    Some(component) => ("Gather", COMPONENTS[component as usize]),
+                    None => ("Sample", ""),
+                };
+                let cmp_str = match depth_ref {
+                    Some(_) => "Cmp",
+                    None => "",
+                };
+                let level_str = match level {
+                    Sl::Zero if gather.is_none() => "LevelZero",
+                    Sl::Auto | Sl::Zero => "",
+                    Sl::Exact(_) => "Level",
+                    Sl::Bias(_) => "Bias",
+                    Sl::Gradient { .. } => "Grad",
                 };
 
                 self.write_expr(module, image, func_ctx)?;
-                write!(self.out, ".{}(", texture_func)?;
+                write!(
+                    self.out,
+                    ".{}{}{}{}(",
+                    base_str, cmp_str, component_str, level_str
+                )?;
                 self.write_expr(module, sampler, func_ctx)?;
                 write!(self.out, ", ")?;
                 self.write_texture_coordinates(
@@ -1949,11 +1942,7 @@ impl<'a, W: fmt::Write> super::Writer<'a, W> {
                 let var = &module.global_variables[var_handle];
                 let (offset, stride) = match module.types[var.ty].inner {
                     TypeInner::Array { stride, .. } => (0, stride),
-                    TypeInner::Struct {
-                        top_level: true,
-                        ref members,
-                        ..
-                    } => {
+                    TypeInner::Struct { ref members, .. } => {
                         let last = members.last().unwrap();
                         let stride = match module.types[last.ty].inner {
                             TypeInner::Array { stride, .. } => stride,
